/*
 *
 *    OPEN-XCHANGE legal information
 *
 *    All intellectual property rights in the Software are protected by
 *    international copyright laws.
 *
 *
 *    In some countries OX, OX Open-Xchange, open xchange and OXtender
 *    as well as the corresponding Logos OX Open-Xchange and OX are registered
 *    trademarks.
 *    The use of the Logos is not covered by the GNU General Public License.
 *    Instead, you are allowed to use these Logos according to the terms and
 *    conditions of the Creative Commons License, Version 2.5, Attribution,
 *    Non-commercial, ShareAlike, and the interpretation of the term
 *    Non-commercial applicable to the aforementioned license is published
 *    on the web site http://www.open-xchange.com/EN/legal/index.html.
 *
 *    Please make sure that third-party modules and libraries are used
 *    according to their respective licenses.
 *
 *    Any modifications to this package must retain all copyright notices
 *    of the original copyright holder(s) for the original code used.
 *
 *    After any such modifications, the original and derivative code shall remain
 *    under the copyright of the copyright holder(s) and/or original author(s)per
 *    the Attribution and Assignment Agreement that can be located at
 *    http://www.open-xchange.com/EN/developer/. The contributing author shall be
 *    given Attribution for the derivative code and a license granting use.
 *
 *     Copyright (C) 2016 OX Software GmbH
 *     Mail: info@open-xchange.com
 *
 *
 *     This program is free software; you can redistribute it and/or modify it
 *     under the terms of the GNU General Public License, Version 2 as published
 *     by the Free Software Foundation.
 *
 *     This program is distributed in the hope that it will be useful, but
 *     WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
 *     or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public License
 *     for more details.
 *
 *     You should have received a copy of the GNU General Public License along
 *     with this program; if not, write to the Free Software Foundation, Inc., 59
 *     Temple Place, Suite 330, Boston, MA 02111-1307 USA
 *
 */

package com.openexchange.office.filter.ooxml.docx.components;

import org.json.JSONObject;
import com.openexchange.office.filter.core.DLNode;
import com.openexchange.office.filter.core.component.ComponentContext;
import com.openexchange.office.filter.core.component.ComponentType;
import com.openexchange.office.filter.core.component.IComponent;
import com.openexchange.office.filter.ooxml.OfficeOpenXMLOperationDocument;
import com.openexchange.office.filter.ooxml.components.OfficeOpenXMLComponent;
import com.openexchange.office.filter.ooxml.components.IShape;
import com.openexchange.office.filter.ooxml.components.IShapeProvider;
import com.openexchange.office.filter.ooxml.components.IShapeType;
import com.openexchange.office.filter.ooxml.components.ShapeType;
import com.openexchange.office.filter.ooxml.docx.DocxOperationDocument;

public class ShapeComponent extends DocxComponent implements IShapeProvider, IShapeType {

	private final IShape shape;

	public ShapeComponent(DocxOperationDocument operationDocument, IShape shape, int _componentNumber) {
		super(operationDocument, new DLNode<Object>(null), _componentNumber);
		this.shape = shape;
	}

	@Override
	public IShape getShape() {
	    return shape;
	}

	@Override
	public ShapeType getType() {
		return shape.getType();
	}
	@Override
	public OfficeOpenXMLComponent getNextComponent() {
	    final IShape next = shape.getNext();
	    return next!=null?new ShapeComponent(getOperationDocument(), next, getComponentNumber()+1):null;
	}
	@Override
	public IComponent<OfficeOpenXMLOperationDocument> getNextChildComponent(ComponentContext<OfficeOpenXMLOperationDocument> previousChildContext, IComponent<OfficeOpenXMLOperationDocument> previousChildComponent) {
	    final Object child = shape.getChild();
	    if(child instanceof IShape) {
	        return new ShapeComponent(getOperationDocument(), (IShape)child, 0);
	    }
	    else if(child instanceof OfficeOpenXMLComponent) {
	        return (OfficeOpenXMLComponent)child;
	    }
	    return null;
	}
	@Override
	public IComponent<OfficeOpenXMLOperationDocument> insertChildComponent(int number, JSONObject attrs, ComponentType type) throws Exception {

	    final Object child = shape.insertChild(number, type);
	    IComponent<OfficeOpenXMLOperationDocument> childComponent = null;
	    if(child instanceof IShape) {
            childComponent = new ShapeComponent(getOperationDocument(), (IShape)child, number);
        }
        else if(child instanceof OfficeOpenXMLComponent) {
            childComponent = (OfficeOpenXMLComponent)child;
        }
	    if(attrs!=null) {
	        childComponent.applyAttrsFromJSON(attrs);
	    }
	    return childComponent;
	}
	@Override
	public void applyAttrsFromJSON(JSONObject attrs) throws Exception {

		shape.applyAttrsFromJSON(attrs);
	}
	@Override
	public JSONObject createJSONAttrs(JSONObject attrs) throws Exception {

		shape.createJSONAttrs(attrs);

		return attrs;
	}
}
